﻿#include "stdafx.h"
#include "acceptor.h"
#include "Session.h"
#include "Listener.h"
#include "SessionManager.h"
#include <MSWSock.h>

/**
@brief 생성자. CListener의 포인터를 받아서 들고 있는다.
*/
CAcceptor::CAcceptor(CListener* pcListener) : m_pcListener(pcListener)
{
}

/**
@brief 소멸자. 별다른 일을 하지 않는다.
*/
CAcceptor::~CAcceptor()
{
	LOG_DEBUG(_T("%s Complete"), __TFUNCTION__);
}

/**
@brief Accept를 다수 거는 메소드.
@param nCount 이 파라미터 만큼 Accept를 걸어둔다.
@desc Accept() 메소드에서 실제 Accept동작이 이루어진다.
*/
void CAcceptor::Accept(int nCount)
{
	for( int i = 0; i < nCount; i++ )
	{
		Accept();
	}
}

/**
@brief Accept를 걸어놓는 메소드.
@return 성공 여부
@desc 실질적인 Accept 동작이 이뤄지는 메소드.
*/
bool CAcceptor::Accept()
{
	CSession* pSession = GetSessionManager()->Pop();
	if(pSession == NULL)
	{
		LOG_ERROR(_T("CAcceptor::Accpet() pSession == NULL"));
		return false;
	}

	pSession->SetSocket(WSASocket(AF_INET, SOCK_STREAM, IPPROTO_TCP, NULL, 0, WSA_FLAG_OVERLAPPED));
	if(INVALID_SOCKET == pSession->GetSocket())
	{
		LOG_ERROR(_T("%s socket() error. [%d]"), __TFUNCTION__, WSAGetLastError());
		return false;
	}

	pSession->SetWorker(m_pcListener.Get());

	int zero = 0;
	setsockopt(pSession->GetSocket(), SOL_SOCKET, SO_SNDBUF, (char*)&zero, sizeof(zero));
	setsockopt(pSession->GetSocket(), SOL_SOCKET, SO_RCVBUF, (char*)&zero, sizeof(zero));

	CIocpContext* pAcceptContext = pSession->GetAcceptIocpContext();
	if(!::AcceptEx(m_pcListener->GetSocket(), pSession->GetSocket(), pAcceptContext->GetHeadBuffer(), 0, ACCEPT_ADDRESS_LENGTH, ACCEPT_ADDRESS_LENGTH, NULL, pAcceptContext->GetOverlappedPtr()))
	{
		if(WSAGetLastError() != ERROR_IO_PENDING && WSAGetLastError() != WSAEWOULDBLOCK)
		{
			LOG_ERROR(_T("%s error [%d]"), __TFUNCTION__, WSAGetLastError());
			GetSessionManager()->Push(pSession);
			return false;
		}
	}
	LOG_DEBUG(_T("%s Success"), __TFUNCTION__);
	return true;
}