﻿#include "stdafx.h"
#include "EnvironmentManager.h"
#include "Define.h"
#include "Event.h"
#include "debug_stream.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////
VOID WINAPI ServiceMain(DWORD argc, LPTSTR* argv)
{
	GetEnvironmentManager()->ServiceMain(argc, argv);
}

VOID WINAPI ServiceHandler(DWORD fdwControl)
{
	GetEnvironmentManager()->ServiceHandler(fdwControl);
}

bool GetDirectory(TCHAR* szFullPath, TCHAR* szBuffer)
{
	int nLen = (int)_tcslen(szFullPath) ;
	TCSNCPY(szBuffer, _tcslen(szBuffer), szFullPath) ;
	for(int i = 0 ; i < nLen ; i++)
	{
		if(szBuffer[nLen - i - 1] ==_T('\\'))
		{
			szBuffer[nLen - i - 1] = 0 ;
			return true ;
		}
	}
	return false;
}

/**
@brief 생성자. 각 변수의 초기화와 서버 구동 설정을 한다.
@param szProgramName 서버 이름
@param dwServiceStartType 서비스 시작 타입
*/
CEnvironmentManager::CEnvironmentManager(const TCHAR* szProgramName, DWORD dwServiceStartType) : m_strProgramName(szProgramName), m_bPause(false), m_hSrv(NULL), 
	m_eRunType(RUNTYPE_NONE), m_dwServiceStartType(dwServiceStartType), m_bActive(false), m_pcEvent(new CEvent)
{
	CLogManager::GetInstancePtr()->SetLogName(szProgramName);
}

/**
@brief 소멸자. 아무일도 하지 않음.
*/
CEnvironmentManager::~CEnvironmentManager()
{
}

/**
@brief 타입에 따른 어플리케이션 초기화 메소드
@param eRunType 어플리케이션 구동 타입
@param argc 파라미터 갯수
@param argv 파라미터 값
*/
void CEnvironmentManager::Init(RUN_TYPE eRunType, int argc, char** argv)
{
	switch(eRunType)
	{
	case RUNTYPE_SERVICE:
		{
			if(1 < argc)
			{
				if(strcmp(argv[1], "/install") == 0)
				{
					TCHAR szPath[MAX_PATH];
					GetModuleFileName( GetModuleHandle(NULL), szPath, MAX_PATH );
					_Install(szPath);
					return;
				}
				if(strcmp(argv[1], "/uninstall") == 0)
				{
					_UnInstall();
					return;
				}
			}
		}
		break;
	}
	_Start(eRunType);
}

/**
@brief 타입에 따른 어플리케이션 시작 메소드
@param eRunType 어플리케이션 구동 타입
*/
void CEnvironmentManager::_Start(RUN_TYPE eRunType)
{
	m_eRunType = eRunType;
	m_bActive = true;

	switch(eRunType)
	{
	case RUNTYPE_CONSOLE:
		{
			_DebugMain();
		}
		break;
	case RUNTYPE_SERVICE:
		{
			//이건 뭥미???
			TCHAR szPath1[MAX_PATH];
			TCHAR szPath2[MAX_PATH];
			GetModuleFileName( GetModuleHandle(NULL), szPath1, MAX_PATH );
			GetDirectory(szPath1, szPath2);
			SetCurrentDirectory(szPath2);
			//이 사이에 있는것의 의미를 알고 싶다 ㄱ-

			SERVICE_TABLE_ENTRY ste[]=
			{
				{const_cast<TCHAR*>(GetProgramName()), ::ServiceMain},
				{NULL,NULL}
			};

			BOOL ret = StartServiceCtrlDispatcher(ste);
			if(!ret)
			{
				LOG_ERROR(_T("StartServiceCtrlDispatcher() Fail. [%d]"), ret);
				return;
			}
		}
		break;
	case RUNTYPE_WINDOW:
		{
			_WindowMain();
		}
		break;
	}
}

/**
@brief 윈도우 모드 동작 메소드
*/
void CEnvironmentManager::_WindowMain()
{
	MSG msg;
	Constructer();
	while (isActive())
	{
		if(PeekMessage(&msg, NULL, 0, 0, PM_REMOVE))
		{
			if (msg.message==WM_QUIT)
				break;

			TranslateMessage(&msg);
			DispatchMessage(&msg);
		}
		else
		{
			Run();
		}
	}
	Destructer();
}

/**
@brief 디버그 모드 동작 메소드
*/
void CEnvironmentManager::_DebugMain()
{
	Constructer();
	while(isActive())
	{
		Run();
	}	
	Destructer();
}

/**
@brief 서비스 모드 동작 메소드
@param argc 매개 변수 갯수
@param argv 매개 변수 값
*/
void CEnvironmentManager::ServiceMain(DWORD argc, LPTSTR* argv)
{
	m_hSrv=RegisterServiceCtrlHandler(GetProgramName(), ::ServiceHandler);
	if(m_hSrv==0)
	{
		LOG_ERROR(_T("RegisterServiceCtrlHandler 실패."));
		return;
	}

	// 서비스가 시작중임을 알린다.
	_SetStatus(SERVICE_START_PENDING);

	m_bPause = false;
	Constructer();
	while(isActive())
	{
		Run();
	}
	Destructer();
}

/**
@brief 서비스 시작 완료 메소드
*/
void CEnvironmentManager::Complete()
{
	if(m_eRunType == RUNTYPE_SERVICE)
		_SetStatus(SERVICE_RUNNING);
}

/**
@brief 서비스 설치 메소드
@return 성공 여부
*/
bool CEnvironmentManager::_Install(const TCHAR* szFilePath)
{
	bool ret;
	SC_HANDLE hScm, hSrv;

	if(_taccess(szFilePath, 0)!=0){
		LOG_ERROR(_T("Not Exist Files"));
		return false;
	}

	hScm = OpenSCManager(NULL, NULL, SC_MANAGER_CREATE_SERVICE);
	if(hScm == NULL){
		LOG_ERROR(_T("Not Opened SCM"));
		return false;
	}

	hSrv = CreateService(hScm, GetProgramName(), GetProgramName(), SERVICE_PAUSE_CONTINUE|SERVICE_CHANGE_CONFIG, SERVICE_WIN32_OWN_PROCESS, 
		m_dwServiceStartType, SERVICE_ERROR_IGNORE, szFilePath, NULL, NULL, NULL, NULL, NULL);
	if(hSrv==NULL)
	{
		LOG_ERROR(_T("Install Failed."));
		ret = false;
	}
	else
	{
		LOG_ERROR(_T("Install Success."));
		CloseServiceHandle(hSrv);
		ret = true;
	}
	CloseServiceHandle(hScm);
	return ret;
}

/**
@brief 서비스 제거 메소드
@return 성공 여부
*/
bool CEnvironmentManager::_UnInstall()
{
	bool ret;
	SC_HANDLE hScm = OpenSCManager(NULL, NULL, SC_MANAGER_CREATE_SERVICE);
	if(hScm == NULL){
		LOG_ERROR(_T("Not Opened SCM."));
		return false;
	}

	SC_HANDLE hSrv = OpenService(hScm, GetProgramName(), SERVICE_ALL_ACCESS);
	if(hSrv==NULL){
		CloseServiceHandle(hScm);
		LOG_ERROR(_T("Service Not Installed."));
		return false;
	}

	SERVICE_STATUS ss;
	QueryServiceStatus(hSrv, &ss);
	if(ss.dwCurrentState != SERVICE_STOPPED){
		ControlService(hSrv, SERVICE_CONTROL_STOP, &ss);
		Sleep(2000);
	}

	if(DeleteService(hSrv))
	{
		LOG_ERROR(_T("Service Delete Success."));
		ret = true;
	}
	else
	{
		LOG_ERROR(_T("Service Delete Failed."));
		ret = false;
	}
	CloseServiceHandle(hSrv);
	CloseServiceHandle(hScm);
	return ret;
}

/**
@brief 서비스 상태 변경 메소드
@param dwState 바꿀 상태
@param dwAccept 서비스가 받을 수 있는 이벤트 상태 설정. 기본적으로 디폴트 파라미터 값을 사용하면 됨.
@return 상태 변경 성공 여부
*/
bool CEnvironmentManager::_SetStatus(DWORD dwState, DWORD dwAccept)
{
	SERVICE_STATUS ss;
	ss.dwServiceType = SERVICE_WIN32_OWN_PROCESS;
	ss.dwCurrentState = dwState;
	ss.dwControlsAccepted=dwAccept;
	ss.dwWin32ExitCode=0;
	ss.dwServiceSpecificExitCode=0;
	ss.dwCheckPoint=0;
	ss.dwWaitHint=0;

	m_nNowState = dwState;
	if(m_hSrv)
	{
		if(!SetServiceStatus(m_hSrv, &ss))
		{
			LOG_ERROR(_T("SetServiceStatus Fail. %d"), dwState);
			return false;
		}
		else
		{
			LOG_INFO(_T("SetServiceStatus Success. %d"), dwState);
			return true;
		}
	}
	return false;
}

/**
@brief 서비스 핸들러. 서비스 상태 변경시 불려지는 메소드다.
@param fdwControl 바꿀 상태
*/
VOID WINAPI CEnvironmentManager::ServiceHandler(DWORD fdwControl)
{
	if(fdwControl == m_nNowState)
		return;

	switch(fdwControl)
	{
	case SERVICE_CONTROL_PAUSE:
		_SetStatus(SERVICE_PAUSE_PENDING);
		m_bPause = true;
		_SetStatus(SERVICE_PAUSED);
		break;
	case SERVICE_CONTROL_CONTINUE:
		_SetStatus(SERVICE_CONTINUE_PENDING);
		m_bPause = false;
		_SetStatus(SERVICE_RUNNING);
		break;
	case SERVICE_CONTROL_STOP:
		_SetStatus(SERVICE_STOP_PENDING);
		Stop();
		break;
	case SERVICE_CONTROL_INTERROGATE:
		break;
	default:
		_SetStatus(m_nNowState);
		break;
	}
}

/**
@brief 예외를 처리하는 메소드.
@param pEx 예외 정보를 담은 포인터
@return 처리 결과 값
*/
int CEnvironmentManager::ExceptionFilter(EXCEPTION_POINTERS* pEx)
{
	static dostream dos(m_strProgramName);
	return exception_filter(dos, pEx);
}