﻿#include "stdafx.h"
#include "EventWorker.h"
#include "EventSelectInterface.h"

/**
@brief EventWorker Thread 함수
@param pEvent EventWorker포인터
@return 수행 결과
*/
DWORD WINAPI OnSocketEvent(LPVOID pEvent)
{
	((CEventWorker*)pEvent)->Proc();
	return 0;
}

/**
@brief 생성자. 쓰레드를 생성하고, 각종 변수를 초기화한다.
*/
CEventWorker::CEventWorker() : m_hThread(INVALID_HANDLE_VALUE), m_bActiveThread(true)
{
	m_hThread = CreateThread(NULL, 0, OnSocketEvent, this, 0, NULL);
	for(int i = WSA_MAXIMUM_WAIT_EVENTS; 0 <= i ; i--)
	{
		m_aryWsaEvent[i] = INVALID_HANDLE_VALUE;
		m_aryEventSelect[i] = NULL;
		m_stl_stack_EmtpyIndex.push(i);
	}
}

/**
@brief 소멸자. 쓰레드를 종료시킨다.
*/
CEventWorker::~CEventWorker()
{
	InterlockedExchange((LONG *)(&m_bActiveThread), (LONG)false);
	if (WAIT_OBJECT_0 == WaitForSingleObject(m_hThread, INFINITE))
		SAFE_CLOSE_HANDLE(m_hThread);
}

/**
@brief 소켓 이벤트 왔는지 검사하는 메소드
@return 수행 결과.
*/
void CEventWorker::Proc()
{
	while(true)
	{
		if(!m_bActiveThread) return;

		DWORD index = WSAWaitForMultipleEvents(WSA_MAXIMUM_WAIT_EVENTS, m_aryWsaEvent, FALSE, INFINITE, FALSE);
		index = index - WSA_WAIT_EVENT_0;

		_Enter();
		STL_MAP_EVENTSELECT_IDX stl_map_EventSelectIdx = m_stl_map_EventSelectIdx;
		for(STL_MAP_EVENTSELECT_IDX::iterator it = stl_map_EventSelectIdx.begin(); it != stl_map_EventSelectIdx.end(); ++it)
		{
			index = WSAWaitForMultipleEvents(1, &m_aryWsaEvent[it->second], TRUE, 0, FALSE);
			if( index < 0 || WSA_WAIT_FAILED==index || WSA_WAIT_TIMEOUT==index ) 
			{
				continue;
			}
			else
			{
				m_aryEventSelect[it->second]->_Proc();
			}
		}
		_Leave();
	}
}

/**
@brief 이벤트 워커에 등록
@param pcEventSelectInterface 이벤트 셀렉트 객체
@return 성공 여부
*/
bool CEventWorker::Register(CEventSelectInterface* pcEventSelectInterface)
{
	if(m_stl_stack_EmtpyIndex.empty())
		return false;

	int nIndex = m_stl_stack_EmtpyIndex.top();
	m_stl_stack_EmtpyIndex.pop();
	m_aryEventSelect[nIndex] = pcEventSelectInterface;
	m_aryWsaEvent[nIndex] = pcEventSelectInterface->_GetEvent();
	_Enter();
	m_stl_map_EventSelectIdx.insert(STL_MAP_EVENTSELECT_IDX::value_type(pcEventSelectInterface, nIndex));
	_Leave();
	return true;
}

/**
@brief 이벤트 워커에서 해제
@param pcEventSelectInterface 이벤트 셀렉트 객체
@return 성공 여부
*/
bool CEventWorker::UnRegister(CEventSelectInterface* pcEventSelectInterface)
{
	_Enter();
	STL_MAP_EVENTSELECT_IDX::iterator it = m_stl_map_EventSelectIdx.find(pcEventSelectInterface);
	if(it == m_stl_map_EventSelectIdx.end())
	{
		_Leave();
		return false;
	}

	int nIndex = it->second;
	m_aryEventSelect[nIndex] = NULL;
	m_aryWsaEvent[nIndex] = INVALID_HANDLE_VALUE;
	m_stl_map_EventSelectIdx.erase(it);
	m_stl_stack_EmtpyIndex.push(nIndex);
	_Leave();
	return true;
}