﻿#include "stdafx.h"
#include "IocpWorker.h"
#include "Session.h"
#include "IocpInterface.h"

/**
@brief 생성자. 
@param hIOCP GQCS에서 사용할 IOCP핸들
*/
CIocpWorker::CIocpWorker(HANDLE hIOCP) : m_hIOCP(hIOCP)
{

}

/**
@brief 소멸자. 스레드를 멈춘다.
*/
CIocpWorker::~CIocpWorker()
{
	Stop();
}

/**
@brief IOCP 루프 메소드
@desc GQCS와 여러 IO동작을 한다.
*/
void CIocpWorker::Act()
{
	BOOL bRet;
	CSession* pcSession = NULL;
	CIocpContext* pIO = NULL;
	CIocpInterface* pcWorker = NULL;
	DWORD dwTransfer;
	while(1)
	{
		bRet = GetQueuedCompletionStatus(m_hIOCP, &dwTransfer, (LPDWORD)&pcSession, (LPOVERLAPPED*)&pIO, INFINITE);
		if(!bRet || (bRet && dwTransfer == 0)) 
		{
			if(pIO)
			{
				LOG_DEBUG(_T("pIO = %p"), pIO);
				if(pIO->GetIOType() == IOTYPE_ACCEPT) //Accept 또는 Connect 신호 온 것!
				{
					LOG_DEBUG(_T("OnAccept [%d] [%d] [%d] [%p, %d]"), WSAGetLastError(), bRet, dwTransfer, pIO, pIO->GetIOType());
					pcSession = pIO->GetSession();
					pcWorker = pcSession->GetWorker();

					if(bRet)
					{
						pcWorker->OnConnected(pcSession);
					}
					else
					{
						pcWorker->OnDisconnect(pcSession);
					}
				}
				else //연결 끊긴겨~
				{
					LOG_INFO(_T("Close Reason [%d] [%d] [%d] [%p, %d]"), WSAGetLastError(), bRet, dwTransfer, pIO, 
						pIO->GetIOType());

					pcWorker = pcSession->GetWorker();
					if(pcWorker)
					{
						pcWorker->OnDisconnect(pcSession);
					}
					else
					{
						LOG_INFO(_T("Already Closed. Close Reason [%d] [%d] [%d] [%p, %d]"), WSAGetLastError(), bRet, dwTransfer, pIO, 
							pIO->GetIOType());
					}
				}
				continue;
			}
			else
			{				
				LOG_INFO(_T("CWorker::WorkThread() pIO == NULL"));
				Stop();
				return;
			}
		}

		//pIO가 NULL이면 IO완료 패킷이 아니라 사용자 메시지다
		if(pcSession->isConnected())
		{
			if(pIO == NULL)
			{
				LOG_ERROR(_T("%s pIO == NULL [%d]"), __TFUNCTION__, dwTransfer);
				continue;
			}

			if(pIO->GetIOType() == IOTYPE_RECV)
			{
				if(!pcSession->OnReceive(dwTransfer, pIO->GetHeadBuffer()))
					continue;

				pcWorker = pcSession->GetWorker();
				if(pcWorker)
				{ 
					pcWorker->OnReceive(pcSession);
				}
			}
			else if(pIO->GetIOType() == IOTYPE_SEND)
			{
				pcWorker = pcSession->GetWorker();
				if(pcWorker)
				{
					pcSession->OnSend();
					pcWorker->OnSend(pcSession);
				}
			}
		}
	}
}

/**
@brief 스레드를 멈춘다. 그리고 IOCP큐도 종료 메시지를 보낸다.
*/
void CIocpWorker::Stop()
{
	__super::Stop();
	PostQueuedCompletionStatus(m_hIOCP, 0, NULL, NULL);
}