﻿#include "stdafx.h"
#include "PacketWorker.h"
#include "PacketQueue.h"
#include "Event.h"

DWORD WINAPI OnPacketEvent(LPVOID pObj)
{
	return CPacketWorker::Proc();
}

std::vector<HANDLE> CPacketWorker::m_stl_vector_ThreadHandle;
BOOL CPacketWorker::m_bActiveThread = TRUE;

EASY::CAutoPtr<STL_QUEUE_PACKET> CPacketWorker::m_stl_queue_Consumer(new STL_QUEUE_PACKET);
EASY::CAutoPtr<STL_QUEUE_PACKET> CPacketWorker::m_stl_queue_Producer(new STL_QUEUE_PACKET);

CEvent CPacketWorker::m_Event;
CCriticalSection CPacketWorker::m_ConsumerLock;
CCriticalSection CPacketWorker::m_ProducerLock;

/**
@brief 생성자. 아무일도 안함.
*/
CPacketWorker::CPacketWorker()
{
	
}

/**
@brief 소멸자. 아무일도 안함.
*/
CPacketWorker::~CPacketWorker()
{
}

/**
@brief 쓰레드 할당. 
@param nThreadCount 할당할 쓰레드 갯수
*/
void CPacketWorker::AllocThread(int nThreadCount)
{
	for(int i = 0; i < nThreadCount; i++)
	{
		m_stl_vector_ThreadHandle.push_back(CreateThread(NULL, 0, OnPacketEvent, NULL, 0, NULL));
	}
}

/**
@brief 쓰레드 해제. 
@desc 모든 스레드를 해제하는 기능만 존재한다.
*/
void CPacketWorker::FreeThread()
{
	InterlockedExchange((LONG *)(&m_bActiveThread), FALSE);
	m_Event.Set();
	for(unsigned int i = 0; i < m_stl_vector_ThreadHandle.size(); i++)
	{
		if (WAIT_OBJECT_0 == WaitForSingleObject(m_stl_vector_ThreadHandle[i], INFINITE))
			SAFE_CLOSE_HANDLE(m_stl_vector_ThreadHandle[i]);
	}

	m_stl_vector_ThreadHandle.clear();
}

/**
@brief 루프 메소드
@return 쓰레드 실행 결과값으로 전달할 값 (무조건 true)
*/
bool CPacketWorker::Proc()
{
	while(m_Event.Wait() != WAIT_TIMEOUT)
	{
		while(!_Empty())
		{
			CPacketQueue* pcPacketQueue = _Pop();
			pcPacketQueue->Proc();
			delete pcPacketQueue;
		}

		if(!m_bActiveThread) //여기서 검사하는 이유는 소멸자에서 Set으로 깨웠을 때, while문 들어가서 남은 DB패킷 다 처리하고 나온 후에 종료되어야 하기 때문.
			break;
	}
	return true;
}

/**
@brief 생산자와 소비자를 스왑한다.
@desc 소비자가 모든걸 소비(empty상태) 한 이후에 불러야한다. 그래야 순서가 어긋나지 않는다.
*/
void CPacketWorker::_Swap()
{
	m_ProducerLock.Enter();
	m_ConsumerLock.Enter();
	STL_QUEUE_PACKET *pTemp = m_stl_queue_Consumer.Release();
	m_stl_queue_Consumer.Reset(m_stl_queue_Producer.Release());
	m_ConsumerLock.Leave();

	m_stl_queue_Producer.Reset(pTemp);
	m_ProducerLock.Leave();
}

/**
@brief 소비자 큐에서 하나 꺼낸다.
@return 꺼낸 패킷
*/
CPacketQueue* CPacketWorker::_Pop()
{
	m_ConsumerLock.Enter();
	if(m_stl_queue_Consumer->empty())
	{
		_Swap();
		if(m_stl_queue_Consumer->size()) //하나라도 있으면
		{
			m_ConsumerLock.Leave();
			return _Pop();
		}
		m_ConsumerLock.Leave();
		LOG_ERROR(_T("%s NULL"), __TFUNCTION__); //깨어났다면 무조건 하나는 있어야 되게끔 짜여져 있다.
		return NULL;
	}
	m_ConsumerLock.Leave();

	m_ConsumerLock.Enter();
	CPacketQueue *pcPacketQueue = m_stl_queue_Consumer->front();
	m_stl_queue_Consumer->pop();
	m_ConsumerLock.Leave();
	return pcPacketQueue;
}

/**
@brief 도착한 패킷 큐에 쌓기
@return 패킷 넣기 결과! 무조건 성공임.
*/
bool CPacketWorker::_Push(CPacketQueue* pcPacketQueue)
{
	m_ProducerLock.Enter();
	m_stl_queue_Producer->push(pcPacketQueue);
	m_ProducerLock.Leave();
	m_Event.Set();
	return true;
}