﻿#include "stdafx.h"
#include "PeerExManager.h"

/**
@brief 생성자. 아무일도 안함
*/
CPeerExManager::CPeerExManager()
{

}

/**
@brief 소멸자. 할당한 피어들 모두 해제
*/
CPeerExManager::~CPeerExManager()
{
	_Clear();
}

/**
@brief 피어를 맵에 등록하는 메소드
@param pcPeer 등록할 피어
*/
void CPeerExManager::Set(CPeerEx* pcPeer)
{
	CScopeCriticalSection cs(&m_CriticalSection);
	m_stl_map_Peer[pcPeer->GetIdx()] = pcPeer;
	LOG_DEBUG(_T("%s OK [%d, %d]"), __TFUNCTION__, pcPeer->GetIdx(), pcPeer->GetAccKey());
}

/**
@brief 피어에서 해제하는 메소드
@param nIdx 키 값
@param nAccKey 누적 키 값
*/
void CPeerExManager::Release(int nIdx, int nAccKey)
{
	CPeerEx* pcPeer = Get(nIdx, nAccKey);
	if(pcPeer)
	{
		_Release(pcPeer);
		pcPeer->DecRefCount(); //소유권 포기의 의미로 참조값 감소.
	}
	else
	{
		LOG_ERROR(_T("%s Fail. pcPeer is not found. [%d, %d]"), __TFUNCTION__, nIdx, nAccKey);
	}
}

/**
@brief 등록된 피어 목록에서 빼주는 메소드
@param pcPeer 뺄 피어
*/
void CPeerExManager::_Release(CPeerEx* pcPeer)
{
	CScopeCriticalSection cs(&m_CriticalSection);
	STL_MAP_PeerEx::iterator it = m_stl_map_Peer.find(pcPeer->GetIdx());
	if(it != m_stl_map_Peer.end())
	{
		m_stl_map_Peer.erase(it);
		pcPeer->DecRefCount(); //릴리즈 할 때 한번 빼준다. 목록에서 빠지면서 참조값 감소.
		LOG_DEBUG(_T("%s erase OK [%d, %d]"), __TFUNCTION__, pcPeer->GetIdx(), pcPeer->GetAccKey());
	}
	else
	{
		LOG_ERROR(_T("%s Fail. [%d, %d]"), __TFUNCTION__, pcPeer->GetIdx(), pcPeer->GetAccKey());
	}
}

/**
@brief 사용중인 모든 피어를 삭제
*/
void CPeerExManager::_Clear()
{
	CScopeCriticalSection cs(&m_CriticalSection);
	STL_MAP_PeerEx::iterator it = m_stl_map_Peer.begin();
	while(it != m_stl_map_Peer.end())
	{
		delete it->second;
		it = m_stl_map_Peer.erase(it);
	}
}

/**
@brief 피어 하나를 찾아서 반환해주는 메소드
@param nIdx 키 값
@param nAccKey 누적 키 값
@return 찾은 피어. 못찾으면 NULL을 반환한다.
*/
CPeerEx* CPeerExManager::Get(int nIdx, int nAccKey)
{
	CScopeCriticalSection cs(&m_CriticalSection);
	STL_MAP_PeerEx::iterator it = m_stl_map_Peer.find(nIdx);
	if(it != m_stl_map_Peer.end())
	{
		CPeerEx* pcPeer = it->second;
		if(pcPeer->GetAccKey() != nAccKey)
		{
			LOG_INFO(_T("%s Fail. peer->GetAccKey() != nAccKey [%d, %d]"), __TFUNCTION__, pcPeer->GetIdx(), pcPeer->GetAccKey());
		}
		else
		{
			pcPeer->IncRefCount(); //유효할 때 리턴.
			return pcPeer;
		}
	}
	else
	{
		if(nIdx != -1 && nAccKey != -1) //둘다 -1일 경우 전달 메시지
		{
			LOG_INFO(_T("%s Fail. Not Find Peer [%d, %d]"), __TFUNCTION__, nIdx, nAccKey);
		}
	}
	return NULL;
}

/**
@brief 사용중인 피어를 돌며 Act함수를 호출해주는 메소드
*/
void CPeerExManager::Act()
{
	CScopeCriticalSection cs(&m_CriticalSection);
	for(STL_MAP_PeerEx::iterator it = m_stl_map_Peer.begin(); it != m_stl_map_Peer.end(); ++it)
	{
		(it->second)->Act();
	}
}

/**
@brief 사용중인 피어를 돌며 브로드 캐스트 한다.
@param nLen Receive된 데이터 길이
@param pData 데이터 포인터
@param nAddLen pAddData의 크기
@param pAddData 기록할 추가 데이터
*/
void CPeerExManager::BroadCast(int nLen, void* pData, int nAddLen, void* pAddData)
{
	CScopeCriticalSection cs(&m_CriticalSection);
	for(STL_MAP_PeerEx::iterator it = m_stl_map_Peer.begin(); it != m_stl_map_Peer.end(); ++it)
	{
		(it->second)->Send(nLen, pData, nAddLen, pAddData);
	}
}

/**
@brief 사용중인 피어중 아무 피어나 임의로 뽑아내는 메소드
@return 임의의 피어. 등록된게 하나도 없을 때만 NULL을 리턴.
*/
CPeerEx* CPeerExManager::GetRandPeer()
{
	CScopeCriticalSection cs(&m_CriticalSection);
	std::vector<CPeerEx*> vList;
	for(STL_MAP_PeerEx::iterator it = m_stl_map_Peer.begin(); it != m_stl_map_Peer.end(); ++it)
		vList.push_back(it->second);

	if(vList.empty())
		return NULL;

	return vList[rand() % vList.size()];
}