﻿#pragma once

#include "DBQueue.h"
#include "Macro.h"

/**
@brief 디비 스레드.
@param lpVoid 디비 스레드 클래스 오브젝트
*/
template<typename T>
DWORD WINAPI DBThread(void *lpVoid)
{
	((CDBThread<T>*)lpVoid)->DBThread();
	return 0;
}

/**
@brief 디비 스레드 클래스. 큐 기능도 한다.
*/
template <typename T>
class CDBThread : public CDBQueue
{
	///쓰레드 핸들
	HANDLE m_hThread;

public:
	/**
	@brief 생성자. 스레드를 생성한다.
	*/
	CDBThread()
	{
		m_hThread = CreateThread(NULL, 0, ::DBThread<T>, this, 0, NULL);
	}

	/**
	@brief 소멸자. 스레드를 멈춘다.
	*/
	virtual ~CDBThread()
	{
		Stop();
		m_pcEvent->Set();
		if (WAIT_OBJECT_0 != WaitForSingleObject( m_hThread, INFINITE))
		{
			SAFE_CLOSE_HANDLE(m_hThread);
		}
	}

	/**
	@brief 루프 메소드
	*/
	void DBThread()
	{
		while(m_pcEvent->Wait() != WAIT_TIMEOUT)
		{
			while(!Empty())
			{
				char *pData = Pop();
				if(pData)
				{
					Proc((T*)pData);
					SAFE_DELETE_ARRAY(pData);
				}
			}

			if(!isActive()) //여기서 검사하는 이유는 소멸자에서 Set으로 깨웠을 때, while문 들어가서 남은 DB패킷 다 처리하고 나온 후에 종료되어야 하기 때문.
				return;
		}
	}
	
	///데이터를 큐에 넣는 기능을 하는 메소드. 순수 가상 함수다.
	virtual void Push(T *data) = 0;

	///큐에 넣었던 것을 꺼내서 처리하는 기능을 하는 메소드. 순수 가상 함수다.
	virtual void Proc(T *data) = 0;
};
