﻿#pragma once

#include "SocketCommon.h"
#include <stack>
#include <map>
#include "CriticalSection.h"

class CEventSelectInterface;

///EventWorker Thread 함수
DWORD WINAPI OnSocketEvent(LPVOID pEventSelectInterface);

/**
@brief EventSelect에서 사용하는 쓰레드 클래스.
*/
class CEventWorker
{
	///쓰레드 핸들
	HANDLE m_hThread;

	///쓰레드 활성화 중인지 여부
	bool m_bActiveThread;

	///WSAEVENT 핸들 배열
	WSAEVENT m_aryWsaEvent[WSA_MAXIMUM_WAIT_EVENTS];

	///CEventSelectInterface의 객체 포인터를 담은 배열
	CEventSelectInterface* m_aryEventSelect[WSA_MAXIMUM_WAIT_EVENTS];

	///인덱스 관리용 스택
	std::stack<int> m_stl_stack_EmtpyIndex;
	
	///key:CEventSelectInterface포인터, value:index
	typedef std::map<CEventSelectInterface*, int> STL_MAP_EVENTSELECT_IDX;

	///CEventSelectInterface포인터로, index를 얻어내기 위한 맵
	STL_MAP_EVENTSELECT_IDX m_stl_map_EventSelectIdx;

	///크리티컬 섹션
	CCriticalSection m_CriticalSection;


public:
	///생성자
	CEventWorker();

	///소멸자
	virtual ~CEventWorker();

	///처리 메소드
	void Proc();

	///등록
	bool Register(CEventSelectInterface* eventselect);

	///등록 해제
	bool UnRegister(CEventSelectInterface* eventselect);

private:
	/**
	@brief 크리티컬 섹션 진입
	*/
	inline void _Enter(){m_CriticalSection.Enter();}

	/**
	@brief 크리티컬 섹션 퇴장
	*/
	inline void _Leave(){m_CriticalSection.Leave();}
};