/*
 * $ HTTP Client
 * ----------------------------------------------------------------------------------------------------------------
 *
 * name :          HTTPClient
 * version tag :   0.1.0
 * description :   HTTP Client using WININET
 * author :        Heo Yongsun ( gooshin@opentown.net )
 * This code distributed under BSD LICENSE STYLE.
 */

#ifndef __HTTP_CLIENT
#define __HTTP_CLIENT

//#include <afxwin.h>
#include <tchar.h>
#include <wininet.h>

// PRE-DEFINED CONSTANTS
#define __DEFAULT_AGENT_NAME		"MERONG(0.9/;p)"

// PRE-DEFINED BUFFER SIZE
#define	__SIZE_HTTP_ARGUMENT_NAME	256
#define __SIZE_HTTP_ARGUMENT_VALUE	1024*64

#define __SIZE_HTTP_BUFFER			100000
#define __SIZE_HTTP_RESPONSE_BUFFER	100000
#define __SIZE_HTTP_HEAD_LINE		2048

#define __SIZE_BUFFER				1024
#define __SIZE_SMALL_BUFFER			256

class CHTTPClient 
{
public:					
	typedef struct __GENERIC_HTTP_ARGUMENT										// ARGUMENTS STRUCTURE
	{							
		TCHAR	szName[__SIZE_HTTP_ARGUMENT_NAME];
		TCHAR	szValue[__SIZE_HTTP_ARGUMENT_VALUE];
		DWORD	dwType;

		operator==(const __GENERIC_HTTP_ARGUMENT &argV)
		{
			return !_tcscmp(szName, argV.szName) && !_tcscmp(szValue, argV.szValue);
		}
	} GenericHTTPArgument;

	enum enumRequestMethod														// REQUEST METHOD
	{															
		RequestUnknown=0,
		RequestGetMethod=1,
		RequestPostMethod=2,
		RequestPostMethodMultiPartsFormData=3
	};

	enum enumTypePostArgument													// ARGUMENTS STRUCTURE
	{																		
		TypeUnknown=0,
		TypeNormal=1,
		TypeBinary=2
	};

	// CONSTRUCTOR & DESTRUCTOR
	CHTTPClient();
	virtual ~CHTTPClient();

	static enumRequestMethod	GetMethod(int nMethod);
	static enumTypePostArgument GetPostArgumentType(int nType);

	// Connection handler	
	BOOL	Connect(LPCTSTR szAddress, LPCTSTR szAgent = __DEFAULT_AGENT_NAME, unsigned short nPort = INTERNET_DEFAULT_HTTP_PORT, LPCTSTR szUserAccount = NULL, LPCTSTR szPassword = NULL);
	BOOL	Close();
	VOID	InitilizePostArguments();

	// HTTP Arguments handler	
	VOID	AddPostArguments(LPCTSTR szName, DWORD nValue);
	VOID	AddPostArguments(LPCTSTR szName, LPCTSTR szValue, BOOL bBinary = FALSE);

	// HTTP Method handler 
	BOOL	Request(LPCTSTR szURL, int nMethod = RequestGetMethod, LPCTSTR szAgent = __DEFAULT_AGENT_NAME);
	BOOL	RequestOfURI(LPCTSTR szURI, int nMethod = RequestGetMethod);
	BOOL	Response(PBYTE pHeaderBuffer, DWORD dwHeaderBufferLength, PBYTE pBuffer, DWORD dwBufferLength, DWORD &dwResultSize);	
	LPCTSTR QueryHTTPResponse();
	LPCTSTR QueryHTTPResponseHeader();	

	// General Handler
	DWORD	GetLastError();
	LPCTSTR GetContentType(LPCTSTR szName);
	VOID	ParseURL(LPCTSTR szURL, LPTSTR szProtocol, LPTSTR szAddress, DWORD &dwPort, LPTSTR szURI);

	
protected:				
	std::vector<GenericHTTPArgument> _vArguments;			// POST ARGUMENTS VECTOR

	TCHAR		_szHTTPResponseHTML[__SIZE_HTTP_BUFFER];	// RECEIVE HTTP BODY
	TCHAR		_szHTTPResponseHeader[__SIZE_HTTP_BUFFER];	// RECEIVE HTTP HEADR

	HINTERNET	_hHTTPOpen;									// internet open handle
	HINTERNET	_hHTTPConnection;							// internet connection hadle
	HINTERNET	_hHTTPRequest;								// internet request hadle

	DWORD		_dwError;									// LAST ERROR CODE
	LPCTSTR		_szHost;									//	 HOST NAME
	DWORD		_dwPort;									//  PORT
	DWORD		_dwFlagSSL;									// HTTPS PROTOCOL REQUEST FLAG

	// HTTP Method handler
	DWORD	ResponseOfBytes(PBYTE pBuffer, DWORD dwSize);
	DWORD	GetPostArguments(LPTSTR szArguments, DWORD dwLength);
	BOOL	RequestPost(LPCTSTR szURI);
	BOOL	RequestPostMultiPartsFormData(LPCTSTR szURI);
	BOOL	RequestGet(LPCTSTR szURI);
	DWORD	AllocMultiPartsFormData(PBYTE &pInBuffer, LPCTSTR szBoundary = "--MULTI-PARTS-FORM-DATA-BOUNDARY-");
	VOID	FreeMultiPartsFormData(PBYTE &pBuffer);
	DWORD	GetMultiPartsFormDataLength();
};

void	PrintLastError( DWORD lastError );

#endif	// #ifndef __HTTP_CLIENT
