﻿#pragma once

#include "Singleton.h"

#include <vector>
#include <list>

#include <windows.h>
#include <algorithm>

#include "Str.h"
#include "DirectoryUtil.h"

/**
@brief 로그를 파일로 남길때 사용할 정보 구조체
*/
struct SLogFileName{
	///로그의 메인 네임
	tstring m_strKeyName;

	///로그의 서브 네임
	tstring m_strSubName;

	/**
	@brief 생성자.
	@param strKeyName 로그의 메인 네임
	@param strSubName 로그의 서브 네임
	*/
	SLogFileName(const tstring& strKeyName, const tstring& strSubName) : m_strKeyName(strKeyName), m_strSubName(strSubName)
	{
	}
};

/**
@brief 검색에 쓰이는 비교 클래스
*/
class CompLogFileName
{
	///로그의 메인 네임
	tstring m_strKeyName;

	///로그의 서브 네임
	tstring m_strSubName;

public:
	/**
	@brief 생성자.
	@param strKeyName 로그의 메인 네임
	@param strSubName 로그의 서브 네임
	*/
	CompLogFileName(const tstring& strKeyName, const tstring& strSubName) : m_strKeyName(strKeyName), m_strSubName(strSubName)
	{
	}

	/**
	@brief 오퍼레이터 재정의
	@param rhs 비교 대상
	@return 문자열을 비교해서, 같으면 true, 다르면 false를 리턴한다.
	*/
	bool operator()(const SLogFileName & rhs) const
	{       
		if(m_strKeyName.compare(rhs.m_strKeyName) == 0)
		{
			if(m_strSubName.compare(rhs.m_strSubName))
			{
				return true;
			}
		}
		return false;
	}
};

class CLogWriter
{
	typedef std::list<SLogFileName> STL_LIST_LOGFILENAME;
	
	///로그가 기록될 디렉토리 경로
	tstring	m_strDirectoryPath;

	///로그의 이름 목록을 담은 List
	STL_LIST_LOGFILENAME m_stl_list_LogName;

public:
	/**
	@brief 소멸자. 아무것도 안함.
	*/
	~CLogWriter(){}

	/**
	@brief 로그 경로 얻어오기
	@return 로그 경로
	*/
	const TCHAR* GetLogPath(){return m_strDirectoryPath.c_str();}

	/**
	@brief 로그를 파일에 쓰는 메소드
	@param strKeyName 로그의 메인 네임
	@param strSubName 로그의 서브 네임
	@param strLog 기록할 로그
	*/
	void Write(const tstring& strKeyName, const tstring& strSubName, const tstring strLog)
	{
		TCHAR szfilename[MAX_PATH];
		SYSTEMTIME time;
		GetLocalTime(&time);

		STL_LIST_LOGFILENAME::iterator it = find_if(m_stl_list_LogName.begin(), m_stl_list_LogName.end(), CompLogFileName(strKeyName, strSubName));
		if( m_stl_list_LogName.end() == it ) //새로 넘어온 로그 이름이면, 새로 생성한다.
		{
			CDirectoryUtil::CreateDirectory(m_strDirectoryPath);
			m_stl_list_LogName.push_back(SLogFileName(strKeyName, strSubName));
		}

		STPRINTF(szfilename, _T("%s\\%s_%s_%04d-%02d-%02d.txt"), m_strDirectoryPath.c_str() , strKeyName.c_str(), strSubName.c_str(), time.wYear, time.wMonth, time.wDay); 

		FILE *fp;
		TFOPEN(fp, szfilename, _T("at"));
		if (fp)
		{
			FTPRINTF(fp,_T("%02d:%02d:%02d\t%s\n"), time.wHour, time.wMinute, time.wSecond, strLog.c_str());
			fclose(fp);
		}
	}

	///싱글턴으로 설정.
	STATIC_SINGLETON_PTR(CLogWriter);

	
private:
	/**
	@brief 생성자. 싱글턴으로만 이용되게 하기위해 private 설정. 로그가 저장될 폴더 설정 읽어오는 역할도 한다.
	*/
	CLogWriter()
	{
		std::locale::global(std::locale("kor")); //유니코드 한글 출력을 위한 것!
		TCHAR szStr[512];
		FILE *fp;
		TFOPEN(fp,_T("cfg\\logpath.cfg"), _T("rt"));
		if(fp) 
		{
			if (_ftscanf(fp, _T("LogPath  = %s\n"), szStr) == 1)
			{
				if(szStr[_tcslen(szStr)] !=_T('\\'))
					TCSCAT(szStr, _T("\\"));

				_SetDirectory(szStr);
				fclose(fp);
				return;
			}
			fclose(fp);
		}

		_SetDirectory(_T("c:\\Log\\"));
	}

	/**
	@brief 디렉토리 설정
	*/
	void _SetDirectory(const tstring& strDirectory)
	{
		m_strDirectoryPath = strDirectory;		
	}
};