﻿#pragma once

#include "SocketCommon.h"
#include <queue>
#include <vector>
#include "CriticalSection.h"
#include "AutoPtr.h"

DWORD WINAPI OnPacketEvent(LPVOID pObj);

class CEvent;
class CPacketQueue;

typedef std::queue<CPacketQueue*> STL_QUEUE_PACKET;

/**
@brief 패킷 처리하는 워커 클래스
@desc 패킷을 처리하려면 쓰레드를 생성해주어야 한다. AllocThread호출시 파라미터로 넘어온 nThreadCount 만큼 스레드가 추가된다.
*/
class CPacketWorker
{
	friend class CPacketQueue;
private:
	///Thread 핸들의 벡터
	static std::vector<HANDLE> m_stl_vector_ThreadHandle;

	///쓰레드 활성화 중인지 여부
	static BOOL m_bActiveThread;

	///소비자 큐
	static EASY::CAutoPtr<STL_QUEUE_PACKET> m_stl_queue_Consumer;

	///생산자 큐
	static EASY::CAutoPtr<STL_QUEUE_PACKET> m_stl_queue_Producer;

	///쓰레드 대기에 사용할 이벤트 객체
	static CEvent m_Event;

	///소비자 락
	static CCriticalSection m_ConsumerLock;

	///생산자 락
	static CCriticalSection m_ProducerLock;

public:
	///생성자
	CPacketWorker();

	///소멸자
	~CPacketWorker();

	///쓰레드 생성에 쓰이는 메소드
	static void AllocThread(int nThreadCount);

	///쓰레드 해제에 사용되는 메소드
	static void FreeThread();

	///CPacketWorker의 루프 메소드
	static bool Proc();

private:
	///패킷 넣기
	static bool _Push(CPacketQueue* pcPacketQueue);

	///생산자, 소비자 스왑
	static void _Swap();

	/**
	@brief 큐가 모두 비었는지 검사
	@return 큐가 모두 비었는지 여부
	*/
	static bool _Empty(){return m_stl_queue_Consumer->empty() && m_stl_queue_Producer->empty();}

	///패킷 하나 꺼내기
	static CPacketQueue* _Pop();
};