﻿#pragma once

#include "Event.h"

///Thread에 사용될 함수의 원형
inline DWORD WINAPI ThreadFunc(void* lpVoid);

/**
@brief Thread를 편리하게 사용하도록 랩핑한 클래스.
*/
class CThread
{
private:
	///활성화 여부
	bool m_bActive;

	///쓰레드 핸들
	HANDLE m_hThread;

public:
	/**
	@brief 소멸자. 쓰레드를 종료 한다.
	*/
	virtual ~CThread()
	{
		Stop();
		WaitForSingleObject(m_hThread, INFINITE);
		CloseHandle(m_hThread);
	}

	/**
	@brief Active상태 동안 Act메소드를 반복 실행한다.
	*/
	inline void Run()
	{
		while(m_bActive)
		{
			Act();
		}
	}

	/**
	@brief 스레드 멈춤
	*/
	virtual void Stop()
	{
		InterlockedExchange((LONG *)(&m_bActive), (LONG)false);
	}

	///반복적으로 불려지는 메소드. 상속 받은 클래스에서 재정의해서 사용해야함.
	virtual void Act(){}


protected:
	/**
	@brief 생성자. 쓰레드를 생성한다. 상속으로만 생성할 수 있도록 protected로 설정했다.
	*/
	CThread() : m_bActive(true)
	{
		m_hThread = CreateThread(NULL, 0, ::ThreadFunc, this, 0, NULL);
	}
};

/**
@brief Thread함수. CThread 클래스의 Run 메소드를 호출해준다.
*/
inline DWORD WINAPI ThreadFunc(void* lpVoid)
{
	((CThread*)lpVoid)->Run();
	return true;
}
